- Renamed prevent-(setting/reading)-cookies to
[privoxy.git] / parsers.c
index 8824d2d..df2ae59 100644 (file)
--- a/parsers.c
+++ b/parsers.c
@@ -1,4 +1,4 @@
-const char parsers_rcs[] = "$Id: parsers.c,v 1.49 2002/03/09 20:03:52 jongfoster Exp $";
+const char parsers_rcs[] = "$Id: parsers.c,v 1.55 2002/05/08 16:01:07 oes Exp $";
 /*********************************************************************
  *
  * File        :  $Source: /cvsroot/ijbswa/current/parsers.c,v $
@@ -14,7 +14,7 @@ const char parsers_rcs[] = "$Id: parsers.c,v 1.49 2002/03/09 20:03:52 jongfoster
  *                   and `server_set_cookie'.
  *
  * Copyright   :  Written by and Copyright (C) 2001 the SourceForge
- *                IJBSWA team.  http://ijbswa.sourceforge.net
+ *                Privoxy team. http://www.privoxy.org/
  *
  *                Based on the Internet Junkbuster originally written
  *                by and Copyright (C) 1997 Anonymous Coders and
@@ -40,6 +40,29 @@ const char parsers_rcs[] = "$Id: parsers.c,v 1.49 2002/03/09 20:03:52 jongfoster
  *
  * Revisions   :
  *    $Log: parsers.c,v $
+ *    Revision 1.55  2002/05/08 16:01:07  oes
+ *    Optimized add_to_iob:
+ *     - Use realloc instead of malloc(), memcpy(), free()
+ *     - Expand to powers of two if possible, to get
+ *       O(log n) reallocs instead of O(n).
+ *     - Moved check for buffer limit here from chat
+ *     - Report failure via returncode
+ *
+ *    Revision 1.54  2002/04/02 15:03:16  oes
+ *    Tiny code cosmetics
+ *
+ *    Revision 1.53  2002/03/26 22:29:55  swa
+ *    we have a new homepage!
+ *
+ *    Revision 1.52  2002/03/24 13:25:43  swa
+ *    name change related issues
+ *
+ *    Revision 1.51  2002/03/13 00:27:05  jongfoster
+ *    Killing warnings
+ *
+ *    Revision 1.50  2002/03/12 01:45:35  oes
+ *    More verbose logging
+ *
  *    Revision 1.49  2002/03/09 20:03:52  jongfoster
  *    - Making various functions return int rather than size_t.
  *      (Undoing a recent change).  Since size_t is unsigned on
@@ -479,7 +502,7 @@ int flush_socket(jb_socket fd, struct client_state *csp)
       return(0);
    }
 
-   if (write_socket(fd, iob->cur, len))
+   if (write_socket(fd, iob->cur, (size_t)len))
    {
       return(-1);
    }
@@ -493,69 +516,74 @@ int flush_socket(jb_socket fd, struct client_state *csp)
  *
  * Function    :  add_to_iob
  *
* Description :  Add content to the buffered page.
  *
  * Parameters  :
  *          1  :  csp = Current client state (buffers, headers, etc...)
  *          2  :  buf = holds the content to be added to the page
  *          3  :  n = number of bytes to be added
  *
- * Returns     :  Number of bytes in the content buffer.
+ * Returns     :  JB_ERR_OK on success, JB_ERR_MEMORY if out-of-memory
+ *                or buffer limit reached.
  *
  *********************************************************************/
-size_t add_to_iob(struct client_state *csp, char *buf, size_t n)
+jb_err add_to_iob(struct client_state *csp, char *buf, int n)
 {
    struct iob *iob = csp->iob;
-   size_t have, need;
+   size_t used, offset, need, want;
    char *p;
 
-   have = iob->eod - iob->cur;
+   if (n <= 0) return JB_ERR_OK;
 
-   if (n <= 0)
-   {
-      return(have);
-   }
+   used   = iob->eod - iob->buf;
+   offset = iob->cur - iob->buf;
+   need   = used + n + 1;
 
-   need = have + n;
-
-   if ((p = (char *)malloc(need + 1)) == NULL)
+   /*
+    * If the buffer can't hold the new data, extend it first.
+    * Use the next power of two if possible, else use the actual need.
+    */
+   if (need > csp->config->buffer_limit)
    {
-      log_error(LOG_LEVEL_FATAL, "malloc() iob failed: %E");
+      log_error(LOG_LEVEL_ERROR, "Buffer limit reached while extending the buffer (iob)");
+      return JB_ERR_MEMORY;
    }
 
-   if (have)
+   if (need > iob->size)
    {
-      /* there is something in the buffer - save it */
-      memcpy(p, iob->cur, have);
-
-      /* replace the buffer with the new space */
-      freez(iob->buf);
-      iob->buf = p;
+      for (want = csp->iob->size ? csp->iob->size : 512; want <= need;) want *= 2;
+      
+      if (want <= csp->config->buffer_limit && NULL != (p = (char *)realloc(iob->buf, want)))
+      {
+         iob->size = want;
+      }
+      else if (NULL != (p = (char *)realloc(iob->buf, need)))
+      {
+         iob->size = need;
+      }
+      else
+      {
+         log_error(LOG_LEVEL_ERROR, "Extending the buffer (iob) failed: %E");
+         return JB_ERR_MEMORY;
+      }
 
-      /* point to the end of the data */
-      p += have;
-   }
-   else
-   {
-      /* the buffer is empty, free it and reinitialize */
-      freez(iob->buf);
+      /* Update the iob pointers */
+      iob->cur = p + offset;
+      iob->eod = p + used;
       iob->buf = p;
    }
 
    /* copy the new data into the iob buffer */
-   memcpy(p, buf, n);
+   memcpy(iob->eod, buf, (size_t)n);
 
    /* point to the end of the data */
-   p += n;
+   iob->eod += n;
 
    /* null terminate == cheap insurance */
-   *p = '\0';
-
-   /* set the pointers to the new values */
-   iob->cur = iob->buf;
-   iob->eod = p;
+   *iob->eod = '\0';
 
-   return(need);
+   return JB_ERR_OK;
 
 }
 
@@ -600,9 +628,9 @@ char *get_header(struct client_state *csp)
 
    iob->cur = p+1;
 
-   if ((q = strchr(ret, '\r'))) *q = '\0';
+   if ((q = strchr(ret, '\r')) != NULL) *q = '\0';
 
-   /* is this a blank linke (i.e. the end of the header) ? */
+   /* is this a blank line (i.e. the end of the header) ? */
    if (*ret == '\0')
    {
       freez(ret);
@@ -1032,7 +1060,7 @@ jb_err client_te(struct client_state *csp, char **header)
  *********************************************************************/
 jb_err client_referrer(struct client_state *csp, char **header)
 {
-   const char * newval;
+   const char *newval;
 
 #ifdef FEATURE_FORCE_LOAD
    /* Since the referrer can include the prefix even
@@ -1119,7 +1147,7 @@ jb_err client_referrer(struct client_state *csp, char **header)
  *********************************************************************/
 jb_err client_uagent(struct client_state *csp, char **header)
 {
-   const char * newval;
+   const char *newval;
 
    if ((csp->action->flags & ACTION_HIDE_USER_AGENT) == 0)
    {
@@ -1191,7 +1219,7 @@ jb_err client_ua(struct client_state *csp, char **header)
  *********************************************************************/
 jb_err client_from(struct client_state *csp, char **header)
 {
-   const char * newval;
+   const char *newval;
 
    if ((csp->action->flags & ACTION_HIDE_FROM) == 0)
    {
@@ -1652,7 +1680,7 @@ jb_err server_set_cookie(struct client_state *csp, char **header)
       int changed = 0;
 
       /* A variable to store the tag we're working on */
-      char * cur_tag;
+      char *cur_tag;
 
       /* Skip "Set-Cookie:" (11 characters) in header */
       cur_tag = *header + 11;
@@ -1667,7 +1695,7 @@ jb_err server_set_cookie(struct client_state *csp, char **header)
       while (*cur_tag)
       {
          /* Find next tag */
-         char * next_tag = strchr(cur_tag, ';');
+         char *next_tag = strchr(cur_tag, ';');
          if (next_tag != NULL)
          {
             /* Skip the ';' character itself */
@@ -1739,7 +1767,7 @@ int strclean(const char *string, const char *substring)
    int hits = 0, len = strlen(substring);
    char *pos, *p;
 
-   while((pos = strstr(string, substring)))
+   while((pos = strstr(string, substring)) != NULL)
    {
       p = pos + len;
       do
@@ -493,69 +516,74 @@ int flush_socket(jb_socket fd, struct client_state *csp)
  * Function    :  add_to_iob
  *
- * Description :  Add content to the buffered page.
+ * Description :  Add content to the buffered page, expanding the
+ *                buffer if necessary.
  *
  * Parameters  :